=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2009-03-23 14:59:43 +0000
+++ bzrlib/testament.py	2011-04-18 14:27:40 +0000
@@ -91,23 +91,30 @@
 
     long_header = 'bazaar-ng testament version 1\n'
     short_header = 'bazaar-ng testament short form 1\n'
+    include_root = False
 
     @classmethod
     def from_revision(cls, repository, revision_id):
-        """Produce a new testament from a historical revision"""
+        """Produce a new testament from a historical revision."""
         rev = repository.get_revision(revision_id)
-        inventory = repository.get_inventory(revision_id)
-        return cls(rev, inventory)
-
-    def __init__(self, rev, inventory):
-        """Create a new testament for rev using inventory."""
+        tree = repository.revision_tree(revision_id)
+        return cls(rev, tree)
+
+    @classmethod
+    def from_revision_tree(cls, tree):
+        """Produce a new testament from a revision tree."""
+        rev = tree._repository.get_revision(tree.get_revision_id())
+        return cls(rev, tree)
+
+    def __init__(self, rev, tree):
+        """Create a new testament for rev using tree."""
         self.revision_id = rev.revision_id
         self.committer = rev.committer
         self.timezone = rev.timezone or 0
         self.timestamp = rev.timestamp
         self.message = rev.message
         self.parent_ids = rev.parent_ids[:]
-        self.inventory = inventory
+        self.tree = tree
         self.revprops = copy(rev.properties)
         if contains_whitespace(self.revision_id):
             raise ValueError(self.revision_id)
@@ -143,9 +150,8 @@
         return [line.encode('utf-8') for line in r]
 
     def _get_entries(self):
-        entries = self.inventory.iter_entries()
-        entries.next()
-        return entries
+        return ((path, ie) for (path, versioned, kind, file_id, ie) in
+                self.tree.list_files(include_root=self.include_root))
 
     def _escape_path(self, path):
         if contains_linebreaks(path):
@@ -209,6 +215,7 @@
 
     long_header = 'bazaar-ng testament version 2.1\n'
     short_header = 'bazaar-ng testament short form 2.1\n'
+    include_root = False
     def _entry_to_line(self, path, ie):
         l = Testament._entry_to_line(self, path, ie)[:-1]
         l += ' ' + ie.revision
@@ -224,8 +231,7 @@
 
     long_header = 'bazaar testament version 3 strict\n'
     short_header = 'bazaar testament short form 3 strict\n'
-    def _get_entries(self):
-        return self.inventory.iter_entries()
+    include_root = True
 
     def _escape_path(self, path):
         if contains_linebreaks(path):

=== modified file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	2010-11-05 20:54:32 +0000
+++ bzrlib/tests/test_testament.py	2011-04-18 14:27:40 +0000
@@ -22,7 +22,11 @@
 
 from bzrlib import osutils
 from bzrlib.tests import SymlinkFeature, TestCaseWithTransport
-from bzrlib.testament import Testament, StrictTestament, StrictTestament3
+from bzrlib.testament import (
+    Testament,
+    StrictTestament,
+    StrictTestament3,
+    )
 from bzrlib.transform import TreeTransform
 
 
@@ -136,10 +140,18 @@
         self.assertEqualDiff(
             self.expected('sample_unicode').encode('utf-8'), t.as_text())
 
+    def test_from_tree(self):
+        tree = self.b.repository.revision_tree('test@user-2')
+        testament = self.testament_class().from_revision_tree(tree)
+        text_1 = testament.as_short_text()
+        text_2 = self.from_revision(self.b.repository,
+                                    'test@user-2').as_short_text()
+        self.assertEqual(text_1, text_2)
+
     def test___init__(self):
         revision = self.b.repository.get_revision('test@user-2')
-        inventory = self.b.repository.get_inventory('test@user-2')
-        testament_1 = self.testament_class()(revision, inventory)
+        tree = self.b.repository.revision_tree('test@user-2')
+        testament_1 = self.testament_class()(revision, tree)
         text_1 = testament_1.as_short_text()
         text_2 = self.from_revision(self.b.repository,
                                     'test@user-2').as_short_text()

=== modified file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	2011-04-18 03:15:03 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-04-18 14:27:40 +0000
@@ -130,6 +130,9 @@
   on ``RepositoryFormat`` rather than a method on ``Repository``.
   (Jelmer Vernooij)
 
+* ``Testament`` now takes a ``tree`` rather than an
+  ``inventory``. (Jelmer Vernooij, #762608)
+
 * ``TestCase.failUnlessExists`` and ``failIfExists`` are deprecated in
   favour of ``assertPathExists`` and ``assertPathDoesNotExist`` 
   respectively.

